searchData={"items":[{"type":"behaviour","doc":"Sections are the widgets which appear your ReflectOS screen, showing information to user.","title":"ReflectOS.Kernel.Section","ref":"ReflectOS.Kernel.Section.html"},{"type":"behaviour","doc":"Sections are a type of `Scenic.Scene` (which in turn is a type of GenServer) which renders a widget on the users ReflectOS Dashboard.  Like all extensions to ReflectOS, developers are also required to implement a set of callbacks which are used to allow run time configuration.  If you plan on publishing your extensions for use by others in the community, you can use these callbacks to create a thoughtful and intuitive configuration experience.\n\nFor the runtime UI behavior, Sections follow many of the same paradigms as `Scenic.Scene` and therefore should be very familiar to for developers experienced building native user interfaces with the [Scenic Framework](https://hexdocs.pm/scenic).  With Scenic, the UI is rendered natively (e.g. not via a webview) - this allows much better performance on the devices typically used for smart mirror projects (i.e. devices with a low-profile form factor and reduced processing power such as the `Raspberry Pi Zero W` and `Zero 2 W`).\n\nThis documentation will cover the basics, but it is highly recommended you refer to Scenic documentation for more details on using the framework to building native user interfaces.","title":"Overview - ReflectOS.Kernel.Section","ref":"ReflectOS.Kernel.Section.html#module-overview"},{"type":"behaviour","doc":"Sections are just modules which `use ReflectOS.Kernel.Section`.  The callbacks and other requirements for a Section fall into two major categories:\n\n1. Configuration experience via the [ReflectOS Console](https://github.com/reflect-os/console) web ui.\n2. Runtime native display rendering on the smart mirror/display","title":"Implementing a Section - ReflectOS.Kernel.Section","ref":"ReflectOS.Kernel.Section.html#module-implementing-a-section"},{"type":"behaviour","doc":"In order to drive the ReflectOS console UI, sections are required to contain an `Ecto.Schema` which represents the available configuration options.  This is typically done using the [`embedded_schema/1`](https://hexdocs.pm/ecto/Ecto.Schema.html#embedded_schema/1) macro, as they are not persisted via `Ecto.Repo`.\n\nNote that embedding schemas in your root schema (e.g. `Ecto.Schema.embeds_many/4`) is **not currently supported**.\n\nAdditionally, sections must implement the following callbacks which are used by the console:\n\n* `c:section_definition/0`\n* `c:section_options/0`\n* `c:changeset/2`","title":"Configuration Experience - ReflectOS.Kernel.Section","ref":"ReflectOS.Kernel.Section.html#module-configuration-experience"},{"type":"behaviour","doc":"In order to render the section to the ReflectOS dashboard on the device screen,\nsections are only required to implement a single callback: `c:init_section/3`.\n\nSections can also optionally implement the `c:validate_section/1` and `c:handle_config_update/2` callbacks, which define section runtime functionality.","title":"Runtime native display - ReflectOS.Kernel.Section","ref":"ReflectOS.Kernel.Section.html#module-runtime-native-display"},{"type":"behaviour","doc":"See below for an example Section which renders a simple timer on the user's\nsmart mirror/display:\n\n    defmodule MyReflectOSExtensions.Sections.Timer do\n      # Note that section passes along any options to the underlying `Scenic.Scene`\n      use ReflectOS.Kernel.Section, has_children: false\n\n      alias __MODULE__\n\n      alias Scenic.Graph\n      import Scenic.Primitives, only: [{:text, 3}]\n\n      import Phoenix.Component, only: [sigil_H: 2]\n\n      import ReflectOS.Kernel.Typography\n      alias ReflectOS.Kernel.Section.Definition\n      alias ReflectOS.Kernel.{OptionGroup, Option}\n      import ReflectOS.Kernel.Primitives, only: [render_section_label: 2]\n\n      #####################################################\n      # Section Configuration\n      #####################################################\n\n      # This schema represents the avaible configuration options\n      # for this section.\n      embedded_schema do\n        field(:show_label?, :boolean, default: false)\n        field(:label, :string)\n        field(:timer_seconds, :integer)\n      end\n\n      # This changeset will be called by the Console UI as users\n      # create and update instances of the section.  Since the schema\n      # above is what will be passed to your section at runtime, this is\n      # a great opportunity to ensure the configuration provided will be valid.\n      @impl true\n      def changeset(%Timer{} = section, params \\ %{}) do\n        # Standard `Ecto.Changeset`, we can cast and validate input as usual.\n        section\n        |> cast(params, [:show_label?, :label, :timer_seconds])\n        |> validate_required([:timer_seconds])\n        |> validate_number(:timer_seconds, greater_than: 0)\n      end\n\n      # This is also used by the ConsoleUI.  It provides the name, icon, and\n      # description of the section. See `ReflectOS.Kernel.Section.Definition`\n      # for more details.\n      @impl true\n      def section_definition(),\n        do: %Definition{\n          name: \"Timer\",\n          icon: \"\"\"\n             \n            <!--!Font Awesome Free 6.6.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license/free Copyright 2024 Fonticons, Inc.-->\n             \n           \n        \"\"\",\n        description: fn assigns ->\n          ~H\"\"\"\n          This section allows you to configure a timer for your ReflectOS dashboard, which will count down to zero.\n           \n           \n          To start a new time after it reaches zero, simply update the sections configuration with a new timer.\n          \"\"\"\n        end\n      }\n\n      # These are the options which will be displayed to the user in the\n      # section configuration form in the Console UI.  Note that the `key`\n      # property must match the field in your Ecto schema.\n      # See the `ReflectOS.Kernel.Option` documentation for more details.\n      @impl true\n      def section_options(),\n        do: [\n          # Option groups allow you to visually group fields together\n          # in the Console UI.  They have no effect on the structure of the\n          # the data in your schema.\n          %OptionGroup{\n            label: \"Label\",\n            options: [\n              %Option{\n                key: :show_label?,\n                label: \"Show Label\",\n                config: %{\n                  type: \"checkbox\"\n                }\n              },\n              %Option{\n                key: :label,\n                label: \"Label Text\",\n                hidden: fn %{show_label?: show_label?} ->\n                  !show_label?\n                end\n              }\n            ]\n          },\n          # Timer\n          %Option{\n            key: :timer_seconds,\n            label: \"Timer (seconds)\",\n            config: %{\n              type: \"tel\"\n            }\n          }\n        ]\n\n      #####################################################\n      # Section Runtime\n      #####################################################\n\n      @impl true\n      def init_section(scene, %Timer{} = section_config, _opts) do\n        # Here we simply store all of the fields in the section's configuration\n        # struct (see the embedded_schema above) in the Scene's `assigns`\n        # and then render the UI.\n        scene =\n          scene\n          |> assign(Map.from_struct(section_config))\n          |> render()\n\n        # As mentioned above, Scene's are Genservers, so we can send messages just like we typically would.\n        Process.send_after(self(), :tick_timer, 1_000)\n\n        # `init_section/3` supports all the returns types supported by `Scenic.Scene.init/3`\n        {:ok, scene}\n      end\n\n      @impl GenServer\n      def handle_info(:tick_timer, scene) do\n        # Gets the `:timer_seconds` from the `Scene.assigns`\n        # (which we set from the config in the `init_section` callback) and subtracts 1\n        timer_seconds = get(scene, :timer_seconds) - 1\n\n        # Assign the updated timer and re-render the scene\n        scene =\n          scene\n          |> assign(:timer_seconds, timer_seconds)\n          |> render()\n\n        # We only want to tick the timer until it reaches zero\n        if timer_seconds > 0 do\n          Process.send_after(self(), :tick_timer, 1_000)\n        end\n\n        # return from the `handle_info/2` like normal.\n        {:noreply, scene}\n      end\n\n      defp render(%Scenic.Scene{} = scene) do\n        # Here we render the graph to either show the remaining time or\n        # that the timer has reached zero.  The `h2()` and `bold()` functions\n        # are helpers provided by `ReflectOS.Kernel.Typography` to help create\n        # a consistent look and feel.\n        graph =\n          case scene.assigns[:timer_seconds] do\n            timer_seconds when timer_seconds > 0 ->\n              Graph.build()\n              |> text(\n                \"Timer will go off in #{timer_seconds} seconds.\",\n                h2()\n                |> bold()\n              )\n\n            _ ->\n              Graph.build()\n              |> text(\n                \"Timer has completed!\",\n                h2()\n                |> bold()\n              )\n          end\n\n        label_config =\n          scene.assigns\n          |> Map.take([:show_label?, :label])\n\n        # Use one of the `ReflectOS.Kernel.Components` to render a label\n        # This MUST be called after the graph is complete.\n        graph =\n          render_section_label(graph, label_config)\n\n        # Note that we use `push_section/2` instead of the typical `Scenic.Scene.push_graph/3`\n        push_section(scene, graph)\n      end\n    end\n\nFor more detailed examples of ReflectOS Sections, see the [ReflectOS Core](https://github.com/Reflect-OS/core/tree/main/lib/sections) sections, which are shipped with the pre-built system firmware.","title":"By Example - ReflectOS.Kernel.Section","ref":"ReflectOS.Kernel.Section.html#module-by-example"},{"type":"behaviour","doc":"As you saw in the example above, one of the goals of ReflectOS is to create a cohesive visual experience for users - that is, there is styling consistency across all the sections displayed at any given time.  One of the goals of ReflectOS is to be the one screen in your home that is not constantly trying to get your attention - it should blend into it's environment, but be available for information when needed.\n\nIf you plan on publishing your extension for others to use, please consider following some basic styling guidelines:\n\n* **Use black, white, and grayscale colors only**.  Since many users will be using your section behind a one-way mirror, white text on a black background generally provides the best contrast (this is the default).  Colors are more easily washed out by the reflection of the surrounding environment.\n\n* **Use font size and weight (e.g. bold, normal, light) to establish an information hierarchy**.  Think about the most important information in your section, which you'd like to be available \"at a glance\", and use large font sizes or bold text or both make it more prominent.  For information with lower priority, use smaller font sizes or light text or both - not everything generall needs to be read from across the room!  Particuarly with a fixed screen size (no scrolling), space is at a premium and using smaller, more compact text wherever possible helps conserve space.\n\n* **Sections define their own width and height**.  Native UIs using Scenic don't have the same flexible layout tools (e.g. flexbox) as HTML.  Thus, sections are responsible for establishing their own width - this means wrapping their own text as needed (see `FontMetrics.wrap/5`), properly sizing images, etc.","title":"Look and Feel Consistency - ReflectOS.Kernel.Section","ref":"ReflectOS.Kernel.Section.html#module-look-and-feel-consistency"},{"type":"behaviour","doc":"* **Provide as many defaults as possible in your `embedded_schema`**.  Ease of use is a first-order concern for ReflectOS, so providing reasonable defaults wherever possible is a great way to make it quick an easy for users to get started with your section while also providing them with a high level of customization.\n\n* **Use `push_section/2` instead of `push_graph`**.  To render a `Scenic.Graph` within a `Scenic.Scene`, one would typically use the `Scenic.Scene.push_graph/2` function.  However, Sections must use `push_section/2` instead to ensure proper positioning within the Layout.\n\n* **To aid with styling, use the helpers provide in this library**.  For more details, see `ReflectOS.Kernel.Typography` and `ReflectOS.Kernel.Components`.\n\n* **Use the `config` field on `ReflectOS.Kernel.Option`**.  This can be used to pass properties directly to the HTML input in the Console UI.  For example, you could use the `placeholder` property to show what the input might look like.\n\n* **You can subscribe to System Settings updates**.  If your section uses values from the System Settings (such as the timezone), you can subscribe to changes so you can update the rendered section in realtime.  See `ReflectOS.Kernel.Settings.System` for more details.","title":"Tips and Tricks - ReflectOS.Kernel.Section","ref":"ReflectOS.Kernel.Section.html#module-tips-and-tricks"},{"type":"callback","doc":"Used to cast and validate input from the user via the ReflectOS console web ui.\n\nCan be used like a standard `Ecto.Changeset`.  Here is the callback from the `Timer` example section above:\n\n    @impl true\n    def changeset(%Timer{} = section, params \\ %{}) do\n      section\n      |> cast(params, [:show_label?, :label, :timer_seconds])\n      |> validate_required([:timer_seconds])\n      |> validate_number(:timer_seconds, greater_than: 0)\n    end","title":"ReflectOS.Kernel.Section.changeset/2","ref":"ReflectOS.Kernel.Section.html#c:changeset/2"},{"type":"callback","doc":"Optional callback for when users update a section's configuration while it's displayed on the ReflectOS dashboard.\n\nThe default behavior when section configuration changes is to restart the section process with the new configuration.  This will likely work in most circumstances, but you can override this behavior if it is desirable (e.g. your scene has a lengthy start up time).","title":"ReflectOS.Kernel.Section.handle_config_update/2","ref":"ReflectOS.Kernel.Section.html#c:handle_config_update/2"},{"type":"callback","doc":"Callback invoked during initialization of the section.\n\nWraps the `c:Scenic.Scene.init/3` callback, and allows the same return values.  The `config` argument will be the populated `Ecto.Schema` defined in your section module.  The `options` argument may contain a `styles` property which can use to pass styles from the ReflectOS layout to your graph if you like.\n\nFrom the `Timer` example above:\n\n    def init_section(scene, %Timer{} = section_config, _opts) do\n      scene =\n        scene\n        |> assign(Map.from_struct(section_config))\n        |> render()\n\n      # As mentioned above, Scene's are Genservers, so we can send messages just like we typically would.\n      Process.send_after(self(), :tick_timer, 1_000)\n\n      {:ok, scene}\n    end","title":"ReflectOS.Kernel.Section.init_section/3","ref":"ReflectOS.Kernel.Section.html#c:init_section/3"},{"type":"function","doc":"Renders the section's graph to the ReflectOS layout.\n\nThis is wrapper around `Scenic.Scene.push_graph/3`, and must be used to push graph as ensures any changes to the size of the section are accomodated by the current layout.","title":"ReflectOS.Kernel.Section.push_section/2","ref":"ReflectOS.Kernel.Section.html#push_section/2"},{"type":"callback","doc":"Provides the `ReflectOS.Kernel.Section.Definition` struct for the section.\n\nThis is used to show your section in the Console UI.  From the `Timer` example above:\n\n    def section_definition(),\n      do: %Definition{\n        name: \"Timer\",\n        icon: \"\"\"\n           \n          <!--!Font Awesome Free 6.6.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license/free Copyright 2024 Fonticons, Inc.-->\n           \n         \n      \"\"\",\n      description: fn assigns ->\n        ~H\"\"\"\n        This section allows you to configure a timer for your ReflectOS dashboard, which will count down to zero.\n         \n         \n        To start a new time after it reaches zero, simply update the sections configuration with a new timer.\n        \"\"\"\n      end\n    }\nNote that the `icon` property is passed in as simple string, while the `description` property takes a function which can be passed to `Phoenix.LiveView`.  This allows you to use html in your description.  Additionally,\nnote that the argument must be called `assigns`.\n\nSee `ReflectOS.Kernel.Section.Definition` for more details.","title":"ReflectOS.Kernel.Section.section_definition/0","ref":"ReflectOS.Kernel.Section.html#c:section_definition/0"},{"type":"callback","doc":"Provides the list of options which can be configured through the ReflectOS console.\n\nFrom the `Timer` example above:\n\n    def section_options(),\n      do: [\n        %OptionGroup{\n          label: \"Label\",\n          options: [\n            %Option{\n              key: :show_label?,\n              label: \"Show Label\",\n              config: %{\n                type: \"checkbox\"\n              }\n            },\n            %Option{\n              key: :label,\n              label: \"Label Text\",\n              hidden: fn %{show_label?: show_label?} ->\n                !show_label?\n              end\n            }\n          ]\n        },\n        # Timer\n        %Option{\n          key: :timer_seconds,\n          label: \"Timer (seconds)\",\n          config: %{\n            type: \"tel\"\n          }\n        }\n      ]\n\nNote that any properties in the `config` map will be passed directly to the HTML input in the Console UI configuration form.\n\nSee `ReflectOS.Kernel.Option` and `ReflectOS.Kernel.OptionGroup` for more information.","title":"ReflectOS.Kernel.Section.section_options/0","ref":"ReflectOS.Kernel.Section.html#c:section_options/0"},{"type":"callback","doc":"Optional callback to validate the section config at runtime.\n\nThis is likely to be rarely used, as sections use the `c:changeset/2` callback to validate the configuration from the user, but is provided as it can be useful during development to ensure configuration your section is receiving matches what you expect.","title":"ReflectOS.Kernel.Section.validate_section/1","ref":"ReflectOS.Kernel.Section.html#c:validate_section/1"},{"type":"type","doc":"","title":"ReflectOS.Kernel.Section.t/0","ref":"ReflectOS.Kernel.Section.html#t:t/0"},{"type":"module","doc":"Holds the section definition struct.","title":"ReflectOS.Kernel.Section.Definition","ref":"ReflectOS.Kernel.Section.Definition.html"},{"type":"module","doc":"* **name** [required]: The name of the section type when it's\ndisplayed in the Console UI.\n* **icon** [required]: An SVG in raw string format which will be displayed\nnext to section in the Console UI.\n* **description** [optional]: A short description of what the section does.\nIf provided, must be a single arity function an argument called `assigns` which returns a compiled `HEEx` component.  The best way to accomplish this is to\nuse the `Phoenix.LiveView.sigil_H/2` macro.\n* **auto_align** [optional, default: false]: A boolean indicating if the\nReflectOS layout should inject styles into the `opts` argument in the\nsection's`init_section` based on where the section appears in the layout.  For\nexample, if your section appears on the right side of the screen, the layout\nmight inject a `text_align: right` style.","title":"Struct Fields - ReflectOS.Kernel.Section.Definition","ref":"ReflectOS.Kernel.Section.Definition.html#module-struct-fields"},{"type":"module","doc":"Here is the `Timer` example from the `ReflectOS.Kernel.Section` documentation:\n\n    %Definition{\n      name: \"Timer\",\n      icon: \"\"\"\n         \n          <!--!Font Awesome Free 6.6.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license/free Copyright 2024 Fonticons, Inc.-->\n           \n         \n      \"\"\",\n      description: fn assigns ->\n        ~H\"\"\"\n        This section allows you to configure a timer for your ReflectOS dashboard, which will count down to zero.\n         \n         \n        To start a new time after it reaches zero, simply update the sections configuration with a new timer.\n        \"\"\"\n      end\n    }","title":"Example - ReflectOS.Kernel.Section.Definition","ref":"ReflectOS.Kernel.Section.Definition.html#module-example"},{"type":"type","doc":"","title":"ReflectOS.Kernel.Section.Definition.t/0","ref":"ReflectOS.Kernel.Section.Definition.html#t:t/0"},{"type":"module","doc":"Used to register one or more `ReflectOS.Kernel.Section`s with the system.\n\nIf you are building an extension library with a new section type,\nthe section module must be registered with the ReflectOS system.\nThis will ensure that it's available for users to create and configure\nsections via the Console UI application.\n\nThe recommended way to accomplish this is to do so by starting a task\nin the root supervisor of your application.\n\nFor example:\n\n    defmodule MyReflectOSExtensions.Application do\n      @moduledoc false\n\n      use Application\n\n      alias ReflectOS.Kernel.Section.Registry, as: SectionRegistry\n\n      @impl true\n      def start(_type, _args) do\n        children = [\n          # Start a task to register ReflectOS extensions\n          {Task, fn -> reflect_os_register() end}\n        ]\n\n        opts = [strategy: :one_for_one, name: MyReflectOSExtensions.Supervisor]\n        Supervisor.start_link(children, opts)\n      end\n\n      defp reflect_os_register() do\n        SectionRegistry.register([\n          MyReflectOSExtensions.Sections.MyNewSection,\n        ])\n      end\n    end","title":"ReflectOS.Kernel.Section.Registry","ref":"ReflectOS.Kernel.Section.Registry.html"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"ReflectOS.Kernel.Section.Registry.child_spec/1","ref":"ReflectOS.Kernel.Section.Registry.html#child_spec/1"},{"type":"function","doc":"Retrieves a list of definitions for all registered sections.","title":"ReflectOS.Kernel.Section.Registry.definitions/0","ref":"ReflectOS.Kernel.Section.Registry.html#definitions/0"},{"type":"function","doc":"Retrieves the list of registered section modules.","title":"ReflectOS.Kernel.Section.Registry.list/0","ref":"ReflectOS.Kernel.Section.Registry.html#list/0"},{"type":"function","doc":"Registers a module with the Section registry.\n\nAccepts either a single module or a list of modules.","title":"ReflectOS.Kernel.Section.Registry.register/1","ref":"ReflectOS.Kernel.Section.Registry.html#register/1"},{"type":"behaviour","doc":"Layouts define the arrangment of sections on the screen.","title":"ReflectOS.Kernel.Layout","ref":"ReflectOS.Kernel.Layout.html"},{"type":"behaviour","doc":"Layouts are a type of `Scenic.Scene` (which in turn is a type of GenServer) which defines the arrangement of sections on the users ReflectOS Dashboard.  For example, the `FourCorners` layouts which ships with ReflectOS allows users to place sections in the top left, top right, bottom left, and bottom right corners of the dashboard.  Layouts are passed their configuration, the screen size, and the sections the user has selected to display and are expected to render those sections to the screen.\n\nLike all extensions to ReflectOS, developers are also required to implement a set of callbacks which are used to allow run time configuration.  If you plan on publishing your extensions for use by others in the community, you can use these callbacks to create a thoughtful and intuitive configuration experience.\n\nFor the runtime UI behavior, Layouts follow many of the same paradigms as `Scenic.Scene` and therefore should be very familiar to for developers experienced building native user interfaces with the [Scenic Framework](https://hexdocs.pm/scenic).  With Scenic, the UI is rendered natively (e.g. not via a webview) - this allows much better performance on the devices typically used for smart mirror projects (i.e. devices with a low-profile form factor and reduced processing power such as the `Raspberry Pi Zero W` and `Zero 2 W`).\n\nThis means that Layouts take on an important responsibility in the ReflecOS\nsystem since the typical tools available in webviews (e.g. `flexbox`) are not\navailable.","title":"Overview - ReflectOS.Kernel.Layout","ref":"ReflectOS.Kernel.Layout.html#module-overview"},{"type":"behaviour","doc":"Layouts define a list of available \"locations\" in their `ReflectOS.Kernel.Layout.Definition`, which are essentially areas of the screen where users can place one or more sections.  Layouts are responsible for adding each section to the layout's graph and using `Scenic.Primitive.Transform.Translate` to ensure the section is located in the right place on the screen.","title":"Layout Locations - ReflectOS.Kernel.Layout","ref":"ReflectOS.Kernel.Layout.html#module-layout-locations"},{"type":"behaviour","doc":"Since a `ReflectOS.Kernel.Section` is just a wrapped `Scenic.Component`,\nLayouts can call `c:Scenic.Component.add_to_graph/3` to render them to the\nthe layouts graph.  For example, your layout might contain the following\nfunction:\n\n    def render_section(%Scenic.Graph{} = graph, %Section{} = section, layout_tracker, x, y) do\n      %{id: section_id, module: section_module} = section\n\n      graph\n      |> section_module.add_to_graph({layout_tracker, section_id}, t: {x, y})\n    end\n\nNote that the `add_to_graph` function must be called with a two-part tuple\nin the format `{layout_tracker, section_id}` as the second argument, where `layout_tracker` is a unique id assigned by the layout used to identify the\nsection and `section_id` is the id of the section being rendered.\n\nNote that you should avoid using the `section_id` as the layout tracker,\nsince users are permitted to add the same section multiple times to the same\nlayout.","title":"Rendering Sections - ReflectOS.Kernel.Layout","ref":"ReflectOS.Kernel.Layout.html#module-rendering-sections"},{"type":"behaviour","doc":"Layouts are just modules which `use ReflectOS.Kernel.Layout`.  The callbacks and other requirements for a Layout fall into two major categories:\n\n1. Configuration experience via the [ReflectOS Console](https://github.com/reflect-os/console) web ui.\n2. Runtime native display rendering on the smart mirror/display","title":"Implementing a Layout - ReflectOS.Kernel.Layout","ref":"ReflectOS.Kernel.Layout.html#module-implementing-a-layout"},{"type":"behaviour","doc":"In order to drive the ReflectOS console UI, layouts are required to contain an `Ecto.Schema` which represents the available configuration options.  This is typically done using the [`embedded_schema/1`](https://hexdocs.pm/ecto/Ecto.Schema.html#embedded_schema/1) macro, as they are not persisted via `Ecto.Repo`.\n\nNote that embedding schemas in your root schema (e.g. `Ecto.Schema.embeds_many/4`) is **not currently supported**.\n\nAdditionally, layouts must implement the following callbacks which are used by the console:\n\n* `c:layout_definition/0`\n* `c:layout_options/0`\n* `c:changeset/2`","title":"Configuration Experience - ReflectOS.Kernel.Layout","ref":"ReflectOS.Kernel.Layout.html#module-configuration-experience"},{"type":"behaviour","doc":"In order to render the layout ReflectOS dashboard, modules are required to implement the following callbacks:\n\n* `c:init_layout/3`\n* `c:handle_section_update/3`\n\n`Layouts` can also optionally implement the following callbacks:\n\n* `c:validate_layout/1`\n* `c:handle_config_update/2`\n* `c:handle_sections_update/2`\n* `c:handle_viewport_update/2`\n\nSee the documentation for each callback below for more details.","title":"Runtime native display - ReflectOS.Kernel.Layout","ref":"ReflectOS.Kernel.Layout.html#module-runtime-native-display"},{"type":"behaviour","doc":"For a complete example of a ReflectOS Layout, see the `FourCorner` layout\nfrom [ReflectOS Core](https://github.com/Reflect-OS/core/tree/main/lib/layouts/four_corners.ex), which is shipped with the pre-built system firmware.","title":"Example - ReflectOS.Kernel.Layout","ref":"ReflectOS.Kernel.Layout.html#module-example"},{"type":"callback","doc":"Used to cast and validate input from the user via the ReflectOS console web ui.\n\nCan be used like a standard `Ecto.Changeset`.  If we had a layout called `SimpleLayout` which defined an `Ecto.Schema` with a `:spacing` field, the\nchangeset callback might look like this:\n\n    @impl true\n    def changeset(%SimpleLayout{} = section, params \\\\ %{}) do\n      section\n      |> cast(params, [:spacing])\n      |> validate_required([:spacing])\n      |> validate_number(:spacing, greater_than: 0)\n    end","title":"ReflectOS.Kernel.Layout.changeset/2","ref":"ReflectOS.Kernel.Layout.html#c:changeset/2"},{"type":"callback","doc":"Optional callback for when a user updates a layouts's configuration while it's displayed on the ReflectOS dashboard.\n\nThe default behavior when layout configuration changes is to restart the layout process with the new configuration.  This will likely work in most circumstances, but you can override this behavior if it is desirable to do so.","title":"ReflectOS.Kernel.Layout.handle_config_update/2","ref":"ReflectOS.Kernel.Layout.html#c:handle_config_update/2"},{"type":"callback","doc":"Required callback to handle updates to sections rendered by the layout.\n\nLayouts must implement this function to handle updates to a section's graph.\nIf a section's dimensions change, it may impact where it or other sections\nin the layout should be located (remember that all elements in `Scenic.Scene` are located on at fixed `x,y` location).  This callback allows layouts to\nadjust the positioning of their sections based on the new section size.\n\nThe `layout_tracker` argument is the unique id assigned to the section by the\nlayout when it calls `c:Scenic.Component.add_to_graph/3`, see the docs on [rendering sections](#module-rendering-sections) above.","title":"ReflectOS.Kernel.Layout.handle_section_update/3","ref":"ReflectOS.Kernel.Layout.html#c:handle_section_update/3"},{"type":"callback","doc":"Optional callback for when a user updates the arrangement of sections in the\nlayout locations.\n\nThe default behavior when this changes is to restart the layout process with the new section arrangement.  This will likely work in most circumstances, but you can override this behavior if it is desirable to do so.","title":"ReflectOS.Kernel.Layout.handle_sections_update/2","ref":"ReflectOS.Kernel.Layout.html#c:handle_sections_update/2"},{"type":"callback","doc":"Optional callback for when a user updates the screen size in the ReflectOS\nsystem settings.\n\nThe default behavior when this changes is to restart the layout process with the new viewport size.  This will likely work in most circumstances, but you can override this behavior if it is desirable to do so.","title":"ReflectOS.Kernel.Layout.handle_viewport_update/2","ref":"ReflectOS.Kernel.Layout.html#c:handle_viewport_update/2"},{"type":"callback","doc":"Callback invoked during initialization of the layout.\n\nWraps the `c:Scenic.Scene.init/3` callback, and allows the same return values.\n\nThe first argument is the `Scenic.Scene`.\n\nThe second argument is a map containing three fields:\n* `config`: The `Ecto.Schema` struct defined in your layout module populated with the user's configuration.\n* `sections`: A `Map` where each key corresponds to a one of the locations\ndefined in `c:layout_definition/0` and values are a list of the `ReflectOS.Kernel.Section` struct.\n* `viewport_size`: A tuple representing the screen size in `{width, height}`\nformat.  The ReflectOS default is `{1080, 1920}` but can be adjusted by user.\n\nThe last argument is a list of options which maybe passed into the layout by the system.  These are not currently used but are included to ensure\nconsistency with `Scenic.Scene`.","title":"ReflectOS.Kernel.Layout.init_layout/3","ref":"ReflectOS.Kernel.Layout.html#c:init_layout/3"},{"type":"callback","doc":"Provides the `ReflectOS.Kernel.Layout.Definition` struct for the layout.\n\nThis is used to show your layout in the Console UI.  See below for an\nexample adapted From the `FourCorner` layout, which ships with\nthe pre-built ReflectOS system:\n\n    @doc false\n    @impl ReflectOS.Kernel.Layout\n    def layout_definition(),\n      do: %Definition{\n        name: \"Four Corner\",\n        description: fn assigns ->\n          ~H\"\"\"\n          Allows placing sections in each of the four corners of the screen, with options for\n          stacking (vertical vs. horizontal) and spacing.\n          \"\"\"\n        end,\n        icon: \"\"\"\n           \n             \n           \n        \"\"\",\n        locations: [\n          %{key: :top_left, label: \"Top Left\"},\n          %{key: :top_right, label: \"Top Right\"},\n          %{key: :bottom_left, label: \"Bottom Left\"},\n          %{key: :bottom_right, label: \"Bottom Right\"}\n        ]\n      }\n\nNote that the `icon` property is passed in as simple string, while the `description` property takes a function which can be passed to `Phoenix.LiveView`.  This allows you to use html tags in your description.  Additionally,\nnote that the argument must be called `assigns`.\n\nThe `locations` property must be a list of maps with a `key` and a `label` property.\n\nSee `ReflectOS.Kernel.Layout.Definition` for more details.","title":"ReflectOS.Kernel.Layout.layout_definition/0","ref":"ReflectOS.Kernel.Layout.html#c:layout_definition/0"},{"type":"callback","doc":"Provides the list of options which can be configured through the ReflectOS console.\n\nGeneral guidelines are to use `ReflectOS.Kernel.OptionGroup` to present\nconfiguration related to a specific layout location together.","title":"ReflectOS.Kernel.Layout.layout_options/0","ref":"ReflectOS.Kernel.Layout.html#c:layout_options/0"},{"type":"callback","doc":"Optional callback to validate the layout config at runtime.\n\nThis is likely to be rarely used, as layouts use the `c:changeset/2` callback to validate the configuration from the user, but is provided as it can be useful during development to ensure the configuration your layout is receiving matches what you expect.","title":"ReflectOS.Kernel.Layout.validate_layout/1","ref":"ReflectOS.Kernel.Layout.html#c:validate_layout/1"},{"type":"type","doc":"","title":"ReflectOS.Kernel.Layout.t/0","ref":"ReflectOS.Kernel.Layout.html#t:t/0"},{"type":"module","doc":"Holds the layout definition struct.","title":"ReflectOS.Kernel.Layout.Definition","ref":"ReflectOS.Kernel.Layout.Definition.html"},{"type":"module","doc":"* **name** [required]: The name of the layout type when it's\ndisplayed in the Console UI.\n* **icon** [required]: An SVG in raw string format which will be displayed\nnext to layout in the Console UI.\n* **description** [optional]: A short description of the layout. If provided,\nmust be a single arity function an argument called `assigns` which returns\na compiled `HEEx` component.  The best way to accomplish this is to\nuse the `Phoenix.LiveView.sigil_H/2` macro.\n* **locations** [required]: A list of locations in the format\n`%{key: :my_location, label: \"MyLocation\"}` indicating where users can place\nsections in your layout.","title":"Struct Fields - ReflectOS.Kernel.Layout.Definition","ref":"ReflectOS.Kernel.Layout.Definition.html#module-struct-fields"},{"type":"module","doc":"Here an example adapted from the `FourCorner` layout in ReflectOS Core:\n\n    %Definition{\n      name: \"Four Corner\",\n      description: fn assigns ->\n        ~H\"\"\"\n        Allows placing sections in each of the four corners of the screen, with options for\n        stacking (vertical vs. horizontal) and spacing.\n        \"\"\"\n      end,\n      icon: \"\"\"\n         \n           \n         \n      \"\"\",\n      locations: [\n        %{key: :top_left, label: \"Top Left\"},\n        %{key: :top_right, label: \"Top Right\"},\n        %{key: :bottom_left, label: \"Bottom Left\"},\n        %{key: :bottom_right, label: \"Bottom Right\"}\n      ]\n    }","title":"Example - ReflectOS.Kernel.Layout.Definition","ref":"ReflectOS.Kernel.Layout.Definition.html#module-example"},{"type":"type","doc":"","title":"ReflectOS.Kernel.Layout.Definition.t/0","ref":"ReflectOS.Kernel.Layout.Definition.html#t:t/0"},{"type":"module","doc":"Used to register one or more `ReflectOS.Kernel.Layout` with the system.\n\nIf you are building an extension library with a new layout type,\nthe layout module must be registered with the ReflectOS system.\nThis will ensure that it's available for users to create and configure\nlayouts via the Console UI application.\n\nThe recommended way to accomplish this is to do so by starting a task\nin the root supervisor of your application.\n\nFor example:\n\n    defmodule MyReflectOSExtensions.Application do\n      @moduledoc false\n\n      use Application\n\n      alias ReflectOS.Kernel.Layout.Registry, as: LayoutRegistry\n\n      @impl true\n      def start(_type, _args) do\n        children = [\n          # Start a task to register ReflectOS extensions\n          {Task, fn -> reflect_os_register() end}\n        ]\n\n        opts = [strategy: :one_for_one, name: MyReflectOSExtensions.Supervisor]\n        Supervisor.start_link(children, opts)\n      end\n\n      defp reflect_os_register() do\n        LayoutRegistry.register([\n          MyReflectOSExtensions.Layouts.MyNewLayout,\n        ])\n      end\n    end","title":"ReflectOS.Kernel.Layout.Registry","ref":"ReflectOS.Kernel.Layout.Registry.html"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"ReflectOS.Kernel.Layout.Registry.child_spec/1","ref":"ReflectOS.Kernel.Layout.Registry.html#child_spec/1"},{"type":"function","doc":"Retrieves a list of definitions for all registered layouts.","title":"ReflectOS.Kernel.Layout.Registry.definitions/0","ref":"ReflectOS.Kernel.Layout.Registry.html#definitions/0"},{"type":"function","doc":"Retrieves the list of registered layout modules.","title":"ReflectOS.Kernel.Layout.Registry.list/0","ref":"ReflectOS.Kernel.Layout.Registry.html#list/0"},{"type":"function","doc":"Registers a module with the Layout registry.\n\nAccepts either a single module or a list of modules.","title":"ReflectOS.Kernel.Layout.Registry.register/1","ref":"ReflectOS.Kernel.Layout.Registry.html#register/1"},{"type":"behaviour","doc":"Layout Managers are responsible for determining which layout should be\nrendered to the dashboard.","title":"ReflectOS.Kernel.LayoutManager","ref":"ReflectOS.Kernel.LayoutManager.html"},{"type":"behaviour","doc":"Layout Managers are specialized GenServers which control which layout should be shown on the dashboard at any given time.  Layout Managers are\na unique feature of ReflectOS, allowing developers to build managers\nwhich can change the layout in response to just about anything (e.g. a\nfixed schedule, weather conditions, etc).  Using Layout Managers, users\ncan ensure their ReflectOS dashhboard shows just what they need, when\nthey need it.\n\nLayouts Managers follow the same pattern of `Scenic.Scene`, and utilize\na dedicated struct (see `ReflectOS.Kernel.LayoutManager.State`) with an\n`assigns` property as their state.  You can use the `assigns` property to\nhold any data in state that you need.  Layout Managers are passed their\nconfiguration as an init argument.\n\nThere are few requirements for LayoutManagers, which allows a high degree\nof flexibility for building custom logic around which layout is displayed.\n`LayoutManagers` simple use the `push_layout/2` function to notify the\nReflectOS system that a new layout should be displayed.\n\nLike all extensions to ReflectOS, developers are also required to implement a set of callbacks which are used to allow run time configuration.  If you plan on publishing your extensions for use by others in the community, you can use these callbacks to create a thoughtful and intuitive configuration experience.","title":"Overview - ReflectOS.Kernel.LayoutManager","ref":"ReflectOS.Kernel.LayoutManager.html#module-overview"},{"type":"behaviour","doc":"Layouts are just modules which `use ReflectOS.Kernel.LayoutManager` and implement it's behavior.  The callbacks and other requirements for a Layout fall into two major categories:\n\n1. Configuration experience via the [ReflectOS Console](https://github.com/reflect-os/console) web ui.\n2. Managing the logic around which layout should be displayed","title":"Implementing a Layout Manager - ReflectOS.Kernel.LayoutManager","ref":"ReflectOS.Kernel.LayoutManager.html#module-implementing-a-layout-manager"},{"type":"behaviour","doc":"In order to drive the ReflectOS console UI, layout managers are required to contain an `Ecto.Schema` which represents the available configuration options.  This is typically done using the [`embedded_schema/1`](https://hexdocs.pm/ecto/Ecto.Schema.html#embedded_schema/1) macro, as they are not persisted via `Ecto.Repo`.\n\nNote that embedding schemas in your root schema (e.g. `Ecto.Schema.embeds_many/4`) is **not currently supported**.\n\nAdditionally, layouts must implement the following callbacks which are used by the console:\n\n* `c:layout_manager_definition/0`\n* `c:layout_manager_options/0`\n* `c:changeset/2`","title":"Configuration Experience - ReflectOS.Kernel.LayoutManager","ref":"ReflectOS.Kernel.LayoutManager.html#module-configuration-experience"},{"type":"behaviour","doc":"In order to determine which layout renders to the ReflectOS dashboard, modules are on required to implement a single callback:\n\n* `c:init_layout_manager/2`\n\nSections can also optionally implement the following callbacks:\n\n* `c:handle_config_update/2`\n\nSee the documentation for each callback below for more details.","title":"Layout Display Logic - ReflectOS.Kernel.LayoutManager","ref":"ReflectOS.Kernel.LayoutManager.html#module-layout-display-logic"},{"type":"behaviour","doc":"Here is the `Static` layout manager which ships with the pre-built\nReflectOS firmware.  It simply allows the user the select a single\nlayout via the console UI and sets it as the active layout.\n\n    defmodule ReflectOS.Core.LayoutManagers.Static do\n      use ReflectOS.Kernel.LayoutManager\n\n      import Phoenix.Component, only: [sigil_H: 2]\n\n      alias ReflectOS.Kernel.Option\n      alias ReflectOS.Kernel.LayoutManager\n      alias ReflectOS.Kernel.LayoutManager.Definition\n      alias ReflectOS.Kernel.LayoutManager.State\n      alias ReflectOS.Kernel.Settings.LayoutStore\n\n      @impl true\n      def layout_manager_definition() do\n        %Definition{\n          name: \"Static Layout\",\n          icon: \"\"\"\n             \n               \n             \n          \"\"\",\n          description: fn assigns ->\n            ~H\"\"\"\n            Allows selecting a single, static layout.\n            \"\"\"\n          end\n        }\n      end\n\n      embedded_schema do\n        field(:layout, :string)\n      end\n\n      @impl true\n      def changeset(%__MODULE__{} = layout_manager, params \\ %{}) do\n        layout_manager\n        |> cast(params, [:layout])\n        |> validate_required([:layout])\n      end\n\n      @impl true\n      def layout_manager_options(),\n        do: [\n          %Option{\n            key: :layout,\n            label: \"Layout\",\n            config: %{\n              type: \"select\",\n              prompt: \"--Select Layout--\",\n              options:\n                LayoutStore.list()\n                |> Enum.map(fn %{name: name, id: id} ->\n                  {name, id}\n                end)\n            }\n          }\n        ]\n\n      @impl LayoutManager\n      def init_layout_manager(%State{} = state, %{layout: layout_id}) do\n        state =\n          state\n          |> push_layout(layout_id)\n\n        {:ok, state}\n      end\n    end","title":"Example - ReflectOS.Kernel.LayoutManager","ref":"ReflectOS.Kernel.LayoutManager.html#module-example"},{"type":"function","doc":"Convenience function to assign a list or map of values into a `ReflectOS.Kernel.LayoutManager.State` struct.\n\n    iex> state = %State{assigns: %{}}\n    iex> assign(state, some_field: \"Some Field\", another_field: \"Another Field\")\n    %State{assigns: %{some_field: \"Some Field\", another_field: \"Another Field\"}}\n\n    iex> state = %State{assigns: %{}}\n    iex> assign(state, %{some_field: \"Some Field\"})\n    %State{assigns: %{some_field: \"Some Field\"}}","title":"ReflectOS.Kernel.LayoutManager.assign/2","ref":"ReflectOS.Kernel.LayoutManager.html#assign/2"},{"type":"function","doc":"Convenience function to assign a value into a `ReflectOS.Kernel.LayoutManager.State` struct.\n\n    iex> state = %State{assigns: %{}}\n    iex> assign(state, :some_field, \"Some Field\")\n    %State{assigns: %{some_field: \"Some Field\"}}","title":"ReflectOS.Kernel.LayoutManager.assign/3","ref":"ReflectOS.Kernel.LayoutManager.html#assign/3"},{"type":"function","doc":"Convenience function to assign a list of new values into a `ReflectOS.Kernel.LayoutManager.State` struct.\n\nOnly values that do not already exist will be assigned.\n\n    iex> state = %State{assigns: %{existing_field: \"Existing Field\"}}\n    iex> assign_new(state, existing_field: \"New Value\", new_field: \"New Field\")\n    %State{assigns: %{existing_field: \"Existing Field\", new_field: \"New Field\"}}\n\n    iex> state = %State{assigns: %{existing_field: \"Existing Field\"}}\n    iex> assign_new(state, %{existing_field: \"New Value\", new_field: \"New Field\"})\n    %State{assigns: %{existing_field: \"Existing Field\", new_field: \"New Field\"}}","title":"ReflectOS.Kernel.LayoutManager.assign_new/2","ref":"ReflectOS.Kernel.LayoutManager.html#assign_new/2"},{"type":"function","doc":"Convenience function to assign a new values into a `ReflectOS.Kernel.LayoutManager.State` struct.\n\nThe value will only be assigned if it does not already exist in the struct.\n\n    iex> state = %State{assigns: %{existing_field: \"Existing Field\"}}\n    iex> assign_new(state, :existing_field, \"New Value\")\n    %State{assigns: %{existing_field: \"Existing Field\"}}\n\n    iex> state = %State{assigns: %{}}\n    iex> assign_new(state, :new_field, \"New Value\")\n    %State{assigns: %{new_field: \"New Value\"}}","title":"ReflectOS.Kernel.LayoutManager.assign_new/3","ref":"ReflectOS.Kernel.LayoutManager.html#assign_new/3"},{"type":"callback","doc":"Used to cast and validate input from the user via the ReflectOS Console UI.\n\nCan be used like a standard `Ecto.Changeset`.  Here is the callback from the example given above:\n\n    @impl true\n    def changeset(%__MODULE__{} = layout_manager, params \\ %{}) do\n      layout_manager\n      |> cast(params, [:layout])\n      |> validate_required([:layout])\n    end","title":"ReflectOS.Kernel.LayoutManager.changeset/2","ref":"ReflectOS.Kernel.LayoutManager.html#c:changeset/2"},{"type":"function","doc":"Convenience function to fetch an assigned value out of a `ReflectOS.Kernel.LayoutManager.State` struct.\n\n    iex> state = %State{assigns: %{some_field: \"some value\"}}\n    iex> fetch(state, :some_field)\n    {:ok, \"some value\"}\n\n    iex> state = %State{assigns: %{some_field: \"some value\"}}\n    iex> fetch(state, :invalid_key)\n    :error","title":"ReflectOS.Kernel.LayoutManager.fetch/2","ref":"ReflectOS.Kernel.LayoutManager.html#fetch/2"},{"type":"function","doc":"Convenience function to get an assigned value out of a `ReflectOS.Kernel.LayoutManager.State` struct.\n\n    iex> state = %State{assigns: %{some_field: \"some value\"}}\n    iex> get(state, :some_field)\n    \"some value\"\n\n    iex> state = %State{assigns: %{some_field: \"some value\"}}\n    iex> get(state, :invalid_key, \"default value\")\n    \"default value\"","title":"ReflectOS.Kernel.LayoutManager.get/3","ref":"ReflectOS.Kernel.LayoutManager.html#get/3"},{"type":"callback","doc":"Optional callback for when users update a layout manager's configuration.\n\nThe default behavior when the layout manager's configuration changes is\nto restart the layout manager process with the new configuration.\nThis will likely work in most circumstances, but you can override this behavior if it is desirable (e.g. your layout manager has a lengthy start\nup time).","title":"ReflectOS.Kernel.LayoutManager.handle_config_update/2","ref":"ReflectOS.Kernel.LayoutManager.html#c:handle_config_update/2"},{"type":"callback","doc":"Callback invoked during initialization of the layout manager.\n\nWraps the `c:GenServer.init/1` callback, and allows the same return values.  The `config` argument will be the populated `Ecto.Schema` defined in your section module.\n\nFrom the `Static` example above:\n\n    @impl LayoutManager\n    def init_layout_manager(%State{} = state, %{layout: layout_id}) do\n      state =\n        state\n        |> push_layout(layout_id)\n\n      {:ok, state}\n    end","title":"ReflectOS.Kernel.LayoutManager.init_layout_manager/2","ref":"ReflectOS.Kernel.LayoutManager.html#c:init_layout_manager/2"},{"type":"callback","doc":"Provides the `ReflectOS.Kernel.LayoutManager.Definition` struct for the layout.\n\nThis is used to show your layout manager in the Console UI.\nHere is the callback from the example above:\n\n    @impl true\n    def layout_manager_definition() do\n      %Definition{\n        name: \"Static Layout\",\n        icon: \"\"\"\n           \n             \n           \n        \"\"\",\n        description: fn assigns ->\n          ~H\"\"\"\n          Allows selecting a single, static layout.\n          \"\"\"\n        end\n      }\n    end\n\nNote that the `icon` property is passed in as simple string, while the `description` property takes a function which can be passed to `Phoenix.LiveView`.  This allows you to use html tags in your description.  Additionally, note that the argument must be called `assigns`.\n\nSee `ReflectOS.Kernel.LayoutManager.Definition` for more details.","title":"ReflectOS.Kernel.LayoutManager.layout_manager_definition/0","ref":"ReflectOS.Kernel.LayoutManager.html#c:layout_manager_definition/0"},{"type":"callback","doc":"Provides the list of options which can be configured through the ReflectOS console.\n\nFrom the example above:\n\n    @impl true\n    def layout_manager_options(),\n      do: [\n        %Option{\n          key: :layout,\n          label: \"Layout\",\n          config: %{\n            type: \"select\",\n            prompt: \"--Select Layout--\",\n            options:\n              LayoutStore.list()\n              |> Enum.map(fn %{name: name, id: id} ->\n                {name, id}\n              end)\n          }\n        }\n      ]\n\nNote that any properties in the `config` map will be passed directly to the HTML input in the Console UI configuration form.\n\nSee `ReflectOS.Kernel.Option` and `ReflectOS.Kernel.OptionGroup` for more information.","title":"ReflectOS.Kernel.LayoutManager.layout_manager_options/0","ref":"ReflectOS.Kernel.LayoutManager.html#c:layout_manager_options/0"},{"type":"function","doc":"Pushes the layout with the provided id to the dashboard.","title":"ReflectOS.Kernel.LayoutManager.push_layout/2","ref":"ReflectOS.Kernel.LayoutManager.html#push_layout/2"},{"type":"type","doc":"","title":"ReflectOS.Kernel.LayoutManager.response_opts/0","ref":"ReflectOS.Kernel.LayoutManager.html#t:response_opts/0"},{"type":"type","doc":"","title":"ReflectOS.Kernel.LayoutManager.t/0","ref":"ReflectOS.Kernel.LayoutManager.html#t:t/0"},{"type":"module","doc":"Holds the layout manager definition struct.","title":"ReflectOS.Kernel.LayoutManager.Definition","ref":"ReflectOS.Kernel.LayoutManager.Definition.html"},{"type":"module","doc":"* **name** [required]: The name of the layout manager type when it's\ndisplayed in the Console UI.\n* **icon** [required]: An SVG in raw string format which will be displayed\nnext to layout managers in the Console UI.\n* **description** [optional]: A short description of what the section does.\nIf provided, must be a single arity function an argument called `assigns` which returns a compiled `HEEx` component.\nThe best way to accomplish this is to use the `Phoenix.LiveView.sigil_H/2` macro.","title":"Struct Fields - ReflectOS.Kernel.LayoutManager.Definition","ref":"ReflectOS.Kernel.LayoutManager.Definition.html#module-struct-fields"},{"type":"module","doc":"Here is the `Static` example from the `ReflectOS.Kernel.LayoutManger` documentation:\n\n    %Definition{\n      name: \"Static Layout\",\n      icon: \"\"\"\n         \n           \n         \n      \"\"\",\n      description: fn assigns ->\n        ~H\"\"\"\n        Allows selecting a single, static layout.\n        \"\"\"\n      end\n    }","title":"Example - ReflectOS.Kernel.LayoutManager.Definition","ref":"ReflectOS.Kernel.LayoutManager.Definition.html#module-example"},{"type":"type","doc":"","title":"ReflectOS.Kernel.LayoutManager.Definition.t/0","ref":"ReflectOS.Kernel.LayoutManager.Definition.html#t:t/0"},{"type":"module","doc":"Used to register one or more `ReflectOS.Kernel.LayoutManager`s with the system.\n\nIf you are building an extension library with a new layout manager type,\nthe layout manager module must be registered with the ReflectOS system.\nThis will ensure that it's available for users to create and configure\nlayout managers via the Console UI application.\n\nThe recommended way to accomplish this is to do so by starting a task\nin the root supervisor of your application.\n\nFor example:\n\n    defmodule MyReflectOSExtensions.Application do\n      @moduledoc false\n\n      use Application\n\n      alias ReflectOS.Kernel.LayoutManager.Registry, as: LayoutManagerRegistry\n\n      @impl true\n      def start(_type, _args) do\n        children = [\n          # Start a task to register ReflectOS extensions\n          {Task, fn -> reflect_os_register() end}\n        ]\n\n        opts = [strategy: :one_for_one, name: MyReflectOSExtensions.Supervisor]\n        Supervisor.start_link(children, opts)\n      end\n\n      defp reflect_os_register() do\n        LayoutManagerRegistry.register([\n          MyReflectOSExtensions.LayoutManagers.MyNewLayoutManager,\n        ])\n      end\n    end","title":"ReflectOS.Kernel.LayoutManager.Registry","ref":"ReflectOS.Kernel.LayoutManager.Registry.html"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"ReflectOS.Kernel.LayoutManager.Registry.child_spec/1","ref":"ReflectOS.Kernel.LayoutManager.Registry.html#child_spec/1"},{"type":"function","doc":"Retrieves a list of definitions for all registered layout managers.","title":"ReflectOS.Kernel.LayoutManager.Registry.definitions/0","ref":"ReflectOS.Kernel.LayoutManager.Registry.html#definitions/0"},{"type":"function","doc":"Retrieves the list of registered layout manager modules.","title":"ReflectOS.Kernel.LayoutManager.Registry.list/0","ref":"ReflectOS.Kernel.LayoutManager.Registry.html#list/0"},{"type":"function","doc":"Registers a module with the LayoutManager registry.\n\nAccepts either a single module or a list of modules.","title":"ReflectOS.Kernel.LayoutManager.Registry.register/1","ref":"ReflectOS.Kernel.LayoutManager.Registry.html#register/1"},{"type":"module","doc":"Holds the struct which is used as the `ReflectOS.Kernel.LayoutManager` state.","title":"ReflectOS.Kernel.LayoutManager.State","ref":"ReflectOS.Kernel.LayoutManager.State.html"},{"type":"type","doc":"","title":"ReflectOS.Kernel.LayoutManager.State.t/0","ref":"ReflectOS.Kernel.LayoutManager.State.html#t:t/0"},{"type":"module","doc":"Defines the struct used by sections, layouts, and layout managers to\npresent configuration options to users in the ReflectOS Console UI.","title":"ReflectOS.Kernel.Option","ref":"ReflectOS.Kernel.Option.html"},{"type":"module","doc":"* **key** [required]: The key of the field in the configuration schema.\n* **label** [required]: Label used for the HTML input in the Console UI.\n* **hidden** [optional]: Function which determines whether or not this\ninput should be displayed.  Takes a single argument, which is a map of\nconfiguration values as the user fills out the configuration form.\n* **config** [required]: A map which is passed directly to the HTML input\nin the Console UI.","title":"Struct Fields - ReflectOS.Kernel.Option","ref":"ReflectOS.Kernel.Option.html#module-struct-fields"},{"type":"module","doc":"","title":"Examples - ReflectOS.Kernel.Option","ref":"ReflectOS.Kernel.Option.html#module-examples"},{"type":"module","doc":"This example shows a minimal option.\n\n    %Option{\n      key: :rss_feed,\n      label: \"RSS Feed Url\"\n    }","title":"Simple Text Input - ReflectOS.Kernel.Option","ref":"ReflectOS.Kernel.Option.html#module-simple-text-input"},{"type":"module","doc":"This example shows a select input - note that the list of options is\npopulated at runtime, not at compile time.  This provides quite a bit of\nflexibility.\n\n    %Option{\n      key: :time_format,\n      label: \"Time Format\",\n      config: %{\n        type: \"select\",\n        options: [\n          {\"12 Hour (6:30 PM)\", \"%-I:%M %p\"},\n          {\"24 Hour (18:30)\", \"%-H:%M\"},\n          {\"System Default [#{case System.time_format() do\n            \"%-I:%M %p\" -> \"12 Hour (6:30 PM)\"\n            \"%-H:%M\" -> \"24 Hour (18:30)\"\n            _ -> \"\"\n            end}]\", \"system\"}\n        ]\n      }\n    }","title":"Select Input - ReflectOS.Kernel.Option","ref":"ReflectOS.Kernel.Option.html#module-select-input"},{"type":"module","doc":"This example shows how to display help text, which is displayed under\nthe input to provide the user more information on how to fill in the field.\n\nNote that `help_text` takes a single arity function which MUST take an argument called\n`assigns` and return the result of a `Phoenix.Component.sigil_H/2`.\n\nThe ReflectOS Console uses the\n[Flowbite CSS Library](https://flowbite.com/docs/getting-started/introduction/)\nfor styling, so feel free to use any of the css classes that it provides!\n\n    %Option{\n      key: :rss_feed,\n      label: \"RSS Feed Url\",\n      help_text: fn assigns ->\n        ~H\"\"\"\n        Web address of the RSS feed.  Some popular examples can be found\n         \n          here\n         .\n        \"\"\"\n      end\n    }","title":"Input with Help Text - ReflectOS.Kernel.Option","ref":"ReflectOS.Kernel.Option.html#module-input-with-help-text"},{"type":"module","doc":"The `hidden` field allows you specify when the input should be hidden\nbased on the current values the user has provided in the configuration\nform.\n\nConsider this common example where the text input for the label is only\nshown if the user elects to show the label.\n\n    %Option{\n      key: :show_label?,\n      label: \"Show Label\",\n      config: %{\n        type: \"checkbox\"\n      }\n    },\n    %Option{\n      key: :label,\n      label: \"Label Text\",\n      hidden: fn %{show_label?: show_label?} ->\n        !show_label?\n      end\n    }","title":"Hiding Inputs - ReflectOS.Kernel.Option","ref":"ReflectOS.Kernel.Option.html#module-hiding-inputs"},{"type":"type","doc":"","title":"ReflectOS.Kernel.Option.t/0","ref":"ReflectOS.Kernel.Option.html#t:t/0"},{"type":"module","doc":"Creates a visual grouping of configuration option inputs in the Console UI.","title":"ReflectOS.Kernel.OptionGroup","ref":"ReflectOS.Kernel.OptionGroup.html"},{"type":"module","doc":"* **label** [required]: Title of the input grouping.\n* **description** [optional]: A single arity function which can be passed to `Phoenix.LiveView`.\n* **options** [optional]: List of `ReflectOS.Kernel.Option` which should be\ndisplayed in the input grouping.\n\nNote that `description` takes a single arity function which MUST take an argument called\n`assigns` and return the result of a `Phoenix.Component.sigil_H/2`.  This allows\nyou to use html markup in the description.  The ReflectOS Console uses the\n[Flowbite CSS Library](https://flowbite.com/docs/getting-started/introduction/)\nfor styling, so feel free to use any of the css classes that it provides!","title":"Struct Fields - ReflectOS.Kernel.OptionGroup","ref":"ReflectOS.Kernel.OptionGroup.html#module-struct-fields"},{"type":"module","doc":"%OptionGroup{\n      label: \"Label\",\n      description: fn assigns ->\n        ~H\"\"\"\n        The label appears over the section, and can be optionally shown or hidden.\n        For more information, see the documentation\n         \n          here\n         .\n        \"\"\"\n      options: [\n        %Option{\n          key: :show_label?,\n          label: \"Show Label\",\n          config: %{\n            type: \"checkbox\"\n          }\n        },\n        %Option{\n          key: :label,\n          label: \"Label Text\",\n          hidden: fn %{show_label?: show_label?} ->\n            !show_label?\n          end\n        }\n      ]\n    }","title":"Example - ReflectOS.Kernel.OptionGroup","ref":"ReflectOS.Kernel.OptionGroup.html#module-example"},{"type":"type","doc":"","title":"ReflectOS.Kernel.OptionGroup.t/0","ref":"ReflectOS.Kernel.OptionGroup.html#t:t/0"},{"type":"module","doc":"This module allows you to retrieve system settings, as well as\nsubscribe to changes.\n\nA common pattern is to default section configuration to use a\nsystem setting (e.g. time format), but allow users to override it\nfor a given section.\n\nIf the default system setting is used, your section can subscribe\nto changes in the setting so the display can be updated in real-time.\nSystem Setting uses the excellent `PropertyTable` library\nunder the hood, and the message sent to your section when you subscribe\nare the standard `PropertyTable.Event` struct.\n\nHere is an example using the `time_format/0` function,\nexcerpted from the date/time section in\n[ReflectOS Core](https://github.com/Reflect-OS/core):\n\n    defmodule ReflectOS.Core.Sections.DateTime do\n      use ReflectOS.Kernel.Section, has_children: false\n\n      alias ReflectOS.Kernel.Settings.System\n      alias ReflectOS.Kernel.Section.Definition\n      alias ReflectOS.Kernel.{OptionGroup, Option}\n\n\n      embedded_schema do\n        # Default time format to the system setting\n        field(:time_format, :string, default: \"system\")\n      end\n\n      def section_options(),\n        do: [\n          %Option{\n            key: :time_format,\n            label: \"Hour Format\",\n            config: %{\n              type: \"select\",\n              options: [\n                {\"12 Hour (6:30 PM)\", \"%-I:%M %p\"},\n                {\"24 Hour (18:30)\", \"%-H:%M\"},\n                {\"System Default\", \"system\"}\n              ]\n            }\n          }\n        ]\n\n      def init_section(scene, %__MODULE__{} = section_config, opts) do\n        time_format =\n          if section_config.time_format == \"system\" do\n            # If we are using the system setting,\n            # subscribe for updates\n            System.subscribe(\"time_format\")\n            System.time_format()\n          else\n            # otherwise, use the user's selection\n            section_config.time_format\n          end\n\n\n        {:ok, assign(scene, time_format: time_format)}\n      end\n\n      # Handle event when the time_format system setting changes.\n      # This will only get called if the select time format for\n      # the section is the \"System Default\"\n      def handle_info(\n            %PropertyTable.Event{\n              property: [\"system\", \"time_format\"],\n              value: time_format\n            },\n            scene\n          ) do\n        scene =\n          scene\n          |> assign(time_format: time_format)\n\n        {:noreply, render(scene)}\n      end\n    end","title":"ReflectOS.Kernel.Settings.System","ref":"ReflectOS.Kernel.Settings.System.html"},{"type":"function","doc":"Subscribes the current process to changes in the system setting.\nWorks just like `PropertyTable.subscribe/2`, and will send the\ncurrent process a `PropertyTable.Event` struct if the subscribed\nsetting is updated.\n\nSee the example above for an example of how to handle this event.\nNote that the `property` field in the struct will be in the\nformat `[\"system\", \" \"]`.","title":"ReflectOS.Kernel.Settings.System.subscribe/1","ref":"ReflectOS.Kernel.Settings.System.html#subscribe/1"},{"type":"function","doc":"Gets the system time format, which is a formatting string\ncompatible with the `Calendar.strftime/3` function.","title":"ReflectOS.Kernel.Settings.System.time_format/0","ref":"ReflectOS.Kernel.Settings.System.html#time_format/0"},{"type":"function","doc":"Gets the system timezone.","title":"ReflectOS.Kernel.Settings.System.timezone/0","ref":"ReflectOS.Kernel.Settings.System.html#timezone/0"},{"type":"function","doc":"Gets the viewport (screen) size formatted as a `{width, height}` tuple.","title":"ReflectOS.Kernel.Settings.System.viewport_size/0","ref":"ReflectOS.Kernel.Settings.System.html#viewport_size/0"},{"type":"type","doc":"\"time_format\"\n  | \"viewport_size\"\n  | \"timezone\"","title":"ReflectOS.Kernel.Settings.System.system_settings_key/0","ref":"ReflectOS.Kernel.Settings.System.html#t:system_settings_key/0"},{"type":"module","doc":"Helper functions for working with `Scenic.Graph`","title":"ReflectOS.Kernel.GraphHelpers","ref":"ReflectOS.Kernel.GraphHelpers.html"},{"type":"function","doc":"Gets the bottom boundary of the graph.   Note that this could be negative.","title":"ReflectOS.Kernel.GraphHelpers.get_bottom_bound/1","ref":"ReflectOS.Kernel.GraphHelpers.html#get_bottom_bound/1"},{"type":"function","doc":"Gets the left boundary of the graph.  Note that this could be negative.","title":"ReflectOS.Kernel.GraphHelpers.get_left_bound/1","ref":"ReflectOS.Kernel.GraphHelpers.html#get_left_bound/1"},{"type":"function","doc":"Gets the right boundary of the graph.  Note that this could be negative.","title":"ReflectOS.Kernel.GraphHelpers.get_right_bound/1","ref":"ReflectOS.Kernel.GraphHelpers.html#get_right_bound/1"},{"type":"function","doc":"Gets the top boundary of the graph.  Note that this could be negative.","title":"ReflectOS.Kernel.GraphHelpers.get_top_bound/1","ref":"ReflectOS.Kernel.GraphHelpers.html#get_top_bound/1"},{"type":"module","doc":"Provides useful helpers for developing section UIs using `Scenic.Scene`\non the ReflectOS platform.","title":"ReflectOS.Kernel.Primitives","ref":"ReflectOS.Kernel.Primitives.html"},{"type":"function","doc":"Renders a QR code on the provided `Scenic.Graph` which encodes the given `content` string.\n\nUses the `QRCodeEx` library.\n\nOptions passed on for generating a QR Code:\n* `:color`\n* `:background_color`\n* `:width`\n\nWe recommend using the default colors, which renders\na standard black and white QR code on a white background.\n\nThe default `:width` is 250px.\n\nThe remaining `opts` are passed on the underlying `Scenic.Primitives.rect/3`","title":"ReflectOS.Kernel.Primitives.qr_code/3","ref":"ReflectOS.Kernel.Primitives.html#qr_code/3"},{"type":"function","doc":"The \"spec\" version of `qr_code/3`, which allows it to be rendered later.\n\nSee `Scenic.Primitives` for more information.","title":"ReflectOS.Kernel.Primitives.qr_code_spec/2","ref":"ReflectOS.Kernel.Primitives.html#qr_code_spec/2"},{"type":"function","doc":"Renders a label for the given  section graph given the common configuration\nfields of `show_label?` and `label`.\n\nNote that the section label **must** be called after the graph is complete.\n\nSee `section_label/3` for available options.","title":"ReflectOS.Kernel.Primitives.render_section_label/3","ref":"ReflectOS.Kernel.Primitives.html#render_section_label/3"},{"type":"function","doc":"Renders a label for the given  section graph given the label text.\n\nNote that the section label **must** be called after the graph is complete.\n\nOptions are:\n* `:width` - A fixed width for the section, defaults to the\nwidth of the section or of the label text, whichever is greater.\n* `align` - The text alignment of the label, allowed values are `:left`,\n`:center`, and `:right`.","title":"ReflectOS.Kernel.Primitives.section_label/3","ref":"ReflectOS.Kernel.Primitives.html#section_label/3"},{"type":"module","doc":"Provides a set of helper functions working with text in\n `Scenic.Scene` for ReflectOS sections.\n\nWe recommend using these helpers to ensure a consistent look and feel\nbetween sections, so that they are visually cohesive when shown together\non the user's dashboard.\n\nFor example:\n\n    graph\n    |> text(\"Here is some text\", [t: {10,10}] |> h5() |> light())","title":"ReflectOS.Kernel.Typography","ref":"ReflectOS.Kernel.Typography.html"},{"type":"function","doc":"Sets the `font` to `:roboto_bold` and returns the style list for\nmethod piping.  `styles` defaults to an empty list so that\nthis method can be used to start piping.\n\n    iex> p() |> bold()\n    [font: :roboto_bold, font_size: 24]\n\n    iex> bold()\n    [font: :roboto_bold]","title":"ReflectOS.Kernel.Typography.bold/1","ref":"ReflectOS.Kernel.Typography.html#bold/1"},{"type":"function","doc":"Extracts the `:font_size` value from the provided `styles`.\n\n    iex> font_size([font_size: 24])\n    24","title":"ReflectOS.Kernel.Typography.font_size/1","ref":"ReflectOS.Kernel.Typography.html#font_size/1"},{"type":"function","doc":"Sets the font size to 80 and returns the style list for\nmethod piping.  `styles` defaults to an empty list so that\nthis method can be used to start piping.\n\n    iex> h1([font: :arial])\n    [font_size: 80, font: :arial]\n\n    iex> h1()\n    [font_size: 80]","title":"ReflectOS.Kernel.Typography.h1/1","ref":"ReflectOS.Kernel.Typography.html#h1/1"},{"type":"function","doc":"Sets the font size to 72 and returns the style list for\nmethod piping.  `styles` defaults to an empty list so that\nthis method can be used to start piping.\n\n    iex> h2([font: :arial])\n    [font_size: 72, font: :arial]\n\n    iex> h2()\n    [font_size: 72]","title":"ReflectOS.Kernel.Typography.h2/1","ref":"ReflectOS.Kernel.Typography.html#h2/1"},{"type":"function","doc":"Sets the font size to 64 and returns the style list for\nmethod piping.  `styles` defaults to an empty list so that\nthis method can be used to start piping.\n\n    iex> h3([font: :arial])\n    [font_size: 64, font: :arial]\n\n    iex> h3()\n    [font_size: 64]","title":"ReflectOS.Kernel.Typography.h3/1","ref":"ReflectOS.Kernel.Typography.html#h3/1"},{"type":"function","doc":"Sets the font size to 56 and returns the style list for\nmethod piping.  `styles` defaults to an empty list so that\nthis method can be used to start piping.\n\n    iex> h4([font: :arial])\n    [font_size: 56, font: :arial]\n\n    iex> h4()\n    [font_size: 56]","title":"ReflectOS.Kernel.Typography.h4/1","ref":"ReflectOS.Kernel.Typography.html#h4/1"},{"type":"function","doc":"Sets the font size to 48 and returns the style list for\nmethod piping.  `styles` defaults to an empty list so that\nthis method can be used to start piping.\n\n    iex> h5([font: :arial])\n    [font_size: 48, font: :arial]\n\n    iex> h5()\n    [font_size: 48]","title":"ReflectOS.Kernel.Typography.h5/1","ref":"ReflectOS.Kernel.Typography.html#h5/1"},{"type":"function","doc":"Sets the font size to 40 and returns the style list for\nmethod piping.  `styles` defaults to an empty list so that\nthis method can be used to start piping.\n\n    iex> h6([font: :arial])\n    [font_size: 40, font: :arial]\n\n    iex> h6()\n    [font_size: 40]","title":"ReflectOS.Kernel.Typography.h6/1","ref":"ReflectOS.Kernel.Typography.html#h6/1"},{"type":"function","doc":"Sets the font size to 32 and returns the style list for\nmethod piping.  `styles` defaults to an empty list so that\nthis method can be used to start piping.\n\n    iex> h7([font: :arial])\n    [font_size: 32, font: :arial]\n\n    iex> h7()\n    [font_size: 32]","title":"ReflectOS.Kernel.Typography.h7/1","ref":"ReflectOS.Kernel.Typography.html#h7/1"},{"type":"function","doc":"Sets the `font` to `:roboto_bold` and returns the style list for\nmethod piping.  `styles` defaults to an empty list so that\nthis method can be used to start piping.\n\n    iex> p() |> light()\n    [font: :roboto_light, font_size: 24]\n\n    iex> light()\n    [font: :roboto_light]","title":"ReflectOS.Kernel.Typography.light/1","ref":"ReflectOS.Kernel.Typography.html#light/1"},{"type":"function","doc":"Sets the font size to 24 and returns the style list for\nmethod piping.  `styles` defaults to an empty list so that\nthis method can be used to start piping.\n\nIntended to be used as the standard non-header font size.\n\n    iex> p([font: :arial])\n    [font_size: 24, font: :arial]\n\n    iex> p()\n    [font_size: 24]","title":"ReflectOS.Kernel.Typography.p/1","ref":"ReflectOS.Kernel.Typography.html#p/1"},{"type":"function","doc":"Wraps the provided text so that it does not exceed the given width.\n\nThe `styles` keyword list must contain a value for `:font_size`.\nThe default is `:roboto`, but this can be overridden by providing\na value for `:font` in the `styles` keyword list.","title":"ReflectOS.Kernel.Typography.wrap/3","ref":"ReflectOS.Kernel.Typography.html#wrap/3"},{"type":"module","doc":"Custom Type to support storing atom module names in an `Ecto.Changeset`.\n\nYou can use this to persist atoms in your module configuration, which is\nparticularly useful if you have multiple providers for a given service\nused to retrieve information for your section.\n\nFor example:\n\n    defmodule MyWeatherSection do\n      use ReflectOS.Kernel.Section\n\n      embedded_schema do\n        field :weather_service_module, Ecto.Module\n      end\n    end\n\nImplements behavior `Ecto.Type`.","title":"ReflectOS.Kernel.Ecto.Module","ref":"ReflectOS.Kernel.Ecto.Module.html"},{"type":"function","doc":"","title":"ReflectOS.Kernel.Ecto.Module.cast/1","ref":"ReflectOS.Kernel.Ecto.Module.html#cast/1"},{"type":"function","doc":"","title":"ReflectOS.Kernel.Ecto.Module.dump/1","ref":"ReflectOS.Kernel.Ecto.Module.html#dump/1"},{"type":"function","doc":"","title":"ReflectOS.Kernel.Ecto.Module.embed_as/1","ref":"ReflectOS.Kernel.Ecto.Module.html#embed_as/1"},{"type":"function","doc":"","title":"ReflectOS.Kernel.Ecto.Module.equal?/2","ref":"ReflectOS.Kernel.Ecto.Module.html#equal?/2"},{"type":"function","doc":"","title":"ReflectOS.Kernel.Ecto.Module.load/1","ref":"ReflectOS.Kernel.Ecto.Module.html#load/1"},{"type":"function","doc":"","title":"ReflectOS.Kernel.Ecto.Module.type/0","ref":"ReflectOS.Kernel.Ecto.Module.html#type/0"},{"type":"extras","doc":"# ReflectOS Kernel\n\n[![Test](https://github.com/Reflect-OS/kernel/actions/workflows/test.yml/badge.svg)](https://github.com/Reflect-OS/kernel/actions/workflows/test.yml)\n[![Hex.pm Version](https://img.shields.io/hexpm/v/reflect_os_kernel.svg?style=flat)](https://hex.pm/packages/reflect_os_kernel)","title":"ReflectOS Kernel","ref":"readme.html"},{"type":"extras","doc":"ReflectOS is the approachable, configurable, and extensible OS for your smart mirror project.  It is designed to allow anyone to easily install, customize, and enjoy a smart mirror/display - no coding or command line usage required!  Instructions for installing on your device and more details about ReflectOS can be found on the [ReflectOS Firmware](https://github.com/Reflect-OS/firmware) project.\n\nThis package provides the foundation for developers in the community to extend the OS by building their own sections, layouts, and layout managers.\n\n> **Warning!**\n> \n> This project is being actively developed, and the API is not currently stable.\n> That said, would love feedback from those using it to build their own ReflectOS\n> extensions on how it can be improved and further stabilized.  Please feel free\n> to open an issue on the \n> [library GitHub](https://github.com/Reflect-OS/kernel/issues) with an suggestions.","title":"About - ReflectOS Kernel","ref":"readme.html#about"},{"type":"extras","doc":"","title":"Extending ReflectOS - ReflectOS Kernel","ref":"readme.html#extending-reflectos"},{"type":"extras","doc":"ReflectOS allows developers to extend it's functionality by creating elixir libraries which register the extension with the system.  New sections, layouts, and layout managers can all be added to the OS via extension libraries (which are just standard elixir packages containing new ReflectOS modules).\n\nThe best way to start building your own extension library is to create \na standard new elixir project using the `mix new` command, for example:\n```\n$ mix new reflect_os_myextensions --sup\n```\nNote that it is a good idea to use the `--sup` option to generate the \napplication calback, this is a great place to register the modules you \nbuild with the ReflectOS system (more on that later).\n\nThen, bring this package to your project by adding `reflect_os_kernel` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:reflect_os_kernel, \"~> 0.10.0\"}\n  ]\nend\n```","title":"Getting Started - ReflectOS Kernel","ref":"readme.html#getting-started"},{"type":"extras","doc":"Creating an extension is essentially just creating a module in your library which implements the proper elixir behavior.  Most extensions are comprised of a set of callbacks which describe both the runtime behavior and configuration experience through the [ReflectOS Console](https://github.com/reflect-os/console).  \n\nSee the [documentation](https://hexdocs.pm/reflect_os_kernel) for `ReflectOS.Kernel.Section`, `ReflectOS.Kernel.Layout`, and `ReflectOS.Kernel.LayoutManager`  for more details on how to get started.\n\nNote that while Layout Managers are built on top of `GenServer`, Sections and Layouts are built on top of `Scenic.Scene` from the [Scenic UI Framework](https://hexdocs.pm/scenic/overview_general.html).  If you are building your own Section or Layout, it is highly recommended you familiarize yourself with Scenic before getting started.","title":"Creating an Extension - ReflectOS Kernel","ref":"readme.html#creating-an-extension"},{"type":"extras","doc":"Once created, extensions modules must be registered with ReflectOS in order to be available in the console web application which allows users to manage their displays.  Best practice is to do so via a supervised task during your application start up (this is why we recommend creating new projects with the `--sup` option).\n\nFor example:\n\n```elixir\ndefmodule MyReflectOSExtensions.Application do\n  @moduledoc false\n\n  use Application\n\n  alias ReflectOS.Kernel.Section.Registry, as: SectionRegistry\n  alias ReflectOS.Kernel.Layout.Registry, as: LayoutRegistry\n  alias ReflectOS.Kernel.LayoutManager.Registry, as: LayoutManagerRegistry\n\n  @impl true\n  def start(_type, _args) do\n    children = [\n      # Start a task to register ReflectOS extensions\n      {Task, fn -> reflect_os_register() end}\n    ]\n\n    opts = [strategy: :one_for_one, name: MyReflectOSExtensions.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\n\n  defp reflect_os_register() do\n    # Sections\n    SectionRegistry.register([\n      MyReflectOSExtensions.Sections.MyNewSection,\n    ])\n\n    # Layouts\n    LayoutRegistry.register([\n      MyReflectOSExtensions.Layouts.MyNewLayout\n    ])\n\n    # Layout Managers\n    LayoutManagerRegistry.register([\n      MyReflectOSExtensions.Layouts.MyNewLayoutManager\n    ])\n  end\nend\n```","title":"Registering - ReflectOS Kernel","ref":"readme.html#registering"},{"type":"extras","doc":"To test your library and create a custom ReflectOS firmware image which includes your extensions, clone the [latest release of the ReflectOS Firmware](https://github.com/Reflect-OS/firmware/releases).  Be sure to checkout a release (e.g. use the `-b` flag), as the `main` branch may not be stable.  For example:\n```\n$ git clone https://github.com/reflect-os/firmware.git my_custom_firmware -b v0.9.0\n```\nBe sure to use the version number of the most recent release!\n\nFrom there, it's as simple as adding your library as a dependency to the firmware's `mix.exs` file!  We recommend starting using the `path` option to refer to a local directory during development (see `Mix.Tasks.Deps` for more information on the various way to reference a dependency).\n\nOnce you've added your extension library as a dependency, follow the getting started instructions in the [Firmware's Readme](https://github.com/Reflect-OS/firmware) to see your project in action!","title":"Adding Extension Libraries to ReflectOS - ReflectOS Kernel","ref":"readme.html#adding-extension-libraries-to-reflectos"},{"type":"extras","doc":"For examples of how to build your own extensions to ReflectOS, check out the [ReflectOS Core](https://github.com/Reflect-OS/core) project.  These are extensions which are shipped with the pre-built ReflectOS firmware.","title":"Examples - ReflectOS Kernel","ref":"readme.html#examples"},{"type":"extras","doc":"Contributions are welcome for this project!  You can \n[open an issue](https://github.com/Reflect-OS/kernel/issues) to report a bug or request\na feature enhancement.  Code contributions are also welcomed, and can be \nsubmitted by forking this repository and creating a pull request.","title":"Contributing - ReflectOS Kernel","ref":"readme.html#contributing"},{"type":"extras","doc":"Full documentation can found on [HexDocs](https://hexdocs.pm/reflect_os_kernel).","title":"Learn More - ReflectOS Kernel","ref":"readme.html#learn-more"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}